@@ -20,6 +20,7 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <string.h>
 
 #include <glib/gi18n.h>
@@ -37,6 +38,8 @@
 #include "main.h"
 #include "pixbuf-utils.h"
 #include "dlg-save-image.h"
+#include "comments.h"
+#include "gth-exif-utils.h"
 
 #define CONVERT_GLADE_FILE "gthumb_convert.glade"
 #define PROGRESS_GLADE_FILE "gthumb_tools.glade"
@@ -98,7 +101,7 @@ static guint         gth_batch_op_signals[LAST_SIGNAL] = { 0 };
 
 
 static void
-free_progress_data (GthBatchOp *bop) 
+free_progress_data (GthBatchOp *bop)
 {
 	g_return_if_fail (bop != NULL);
 
@@ -161,7 +164,7 @@ free_progress_data (GthBatchOp *bop)
 }
 
 
-static void 
+static void
 gth_batch_op_finalize (GObject *object)
 {
 	GthBatchOp *bop;
@@ -199,7 +202,7 @@ gth_batch_op_class_init (GthBatchOpClass *class)
 			      G_STRUCT_OFFSET (GthBatchOpClass, batch_op_progress),
 			      NULL, NULL,
 			      gthumb_marshal_VOID__FLOAT,
-			      G_TYPE_NONE, 
+			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_FLOAT);
 	gth_batch_op_signals[BOP_DONE] =
@@ -209,7 +212,7 @@ gth_batch_op_class_init (GthBatchOpClass *class)
 			      G_STRUCT_OFFSET (GthBatchOpClass, batch_op_done),
 			      NULL, NULL,
 			      gthumb_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE, 
+			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_BOOLEAN);
 
@@ -258,12 +261,12 @@ gth_batch_op_new (GthPixbufOp *pixbuf_op,
 		  gpointer     extra_data)
 {
 	GthBatchOp *bop;
-	
+
 	g_return_val_if_fail (pixbuf_op != NULL, NULL);
-	
+
 	bop = GTH_BATCH_OP (g_object_new (GTH_TYPE_BATCH_OP, NULL));
 	gth_batch_op_set_op (bop, pixbuf_op, extra_data);
-	
+
 	return bop;
 }
 
@@ -275,10 +278,10 @@ static void pixbuf_op_done_cb (GthPixbufOp *pixop,
 
 static void
 pixbuf_op_progress_cb (GthPixbufOp *pixop,
-		       float        p, 
+		       float        p,
 		       GthBatchOp  *bop)
 {
-	if (PD(bop)->progress_dlg != NULL) 
+	if (PD(bop)->progress_dlg != NULL)
 		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (PD(bop)->image_progressbar), p);
 }
 
@@ -313,7 +316,7 @@ gth_batch_op_set_op (GthBatchOp  *bop,
 
 
 static void
-stop_operation_cb (GtkWidget  *widget, 
+stop_operation_cb (GtkWidget  *widget,
 		   GthBatchOp *bop)
 {
 	g_return_if_fail (GTH_IS_BATCH_OP (bop));
@@ -383,16 +386,16 @@ load_current_image (GthBatchOp *bop)
 		folder = g_strdup (PD(bop)->destination);
 	name_no_ext = remove_extension_from_path (file_name_from_path (fd->path));
 
-	file_mime_type = get_file_mime_type (fd->path, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE)); 
+	file_mime_type = get_file_mime_type (fd->path, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE));
 	if (strcmp_null_tollerant (file_mime_type, get_mime_type_from_ext (PD(bop)->image_type)) == 0)
-		PD(bop)->new_path = g_strconcat (folder, "/", file_name_from_path (fd->path), NULL); 
+		PD(bop)->new_path = g_strconcat (folder, "/", file_name_from_path (fd->path), NULL);
 	else
 		PD(bop)->new_path = g_strconcat (folder, "/", name_no_ext, ".", PD(bop)->image_type, NULL);
 
 	g_free (folder);
 	g_free (name_no_ext);
 
-	utf8_name = g_filename_display_basename (PD(bop)->new_path);
+	utf8_name = basename_for_display (PD(bop)->new_path);
 	message = g_strdup_printf (_("Converting image: %s"), utf8_name);
 
 	gtk_label_set_text (GTK_LABEL (PD(bop)->progress_label), message);
@@ -402,8 +405,8 @@ load_current_image (GthBatchOp *bop)
 
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (PD(bop)->overall_progressbar),
 				       (double) PD(bop)->image / PD(bop)->images);
-	
-	image_loader_set_path (PD(bop)->loader, fd->path);
+
+	image_loader_set_path (PD(bop)->loader, fd->path, NULL);
 	image_loader_start (PD(bop)->loader);
 }
 
@@ -414,13 +417,13 @@ show_rename_dialog (GthBatchOp *bop)
 	char  *message;
 	char  *utf8_name;
 
-	utf8_name = g_filename_display_basename (PD(bop)->new_path);
+	utf8_name = basename_for_display (PD(bop)->new_path);
 
 	message = g_strdup_printf (_("An image named \"%s\" is already present. " "Please specify a different name."), utf8_name);
-	
+
 	_gtk_label_set_locale_text (GTK_LABEL (PD(bop)->conv_ren_message_label), message);
 	g_free (message);
-	
+
 	gtk_entry_set_text (GTK_ENTRY (PD(bop)->conv_ren_name_entry), utf8_name);
 
 	g_free (utf8_name);
@@ -436,7 +439,8 @@ pixbuf_op_done_cb (GthPixbufOp *pixop,
 		   gboolean     completed,
 		   GthBatchOp  *bop)
 {
-	GError *error = NULL;
+	GError   *error = NULL;
+	FileData *fd = PD(bop)->current_image->data;
 
 	if (!completed) {
 		notify_termination (bop);
@@ -444,19 +448,22 @@ pixbuf_op_done_cb (GthPixbufOp *pixop,
 	}
 
 	if (_gdk_pixbuf_savev (pixop->dest,
-			       PD(bop)->new_path, 
+			       PD(bop)->new_path,
 			       PD(bop)->image_type,
-			       PD(bop)->keys, 
+			       PD(bop)->keys,
 			       PD(bop)->values,
 			       &error)) {
-		FileData *fd = PD(bop)->current_image->data;
 		PD(bop)->saved_list = g_list_prepend (PD(bop)->saved_list, g_strdup (PD(bop)->new_path));
 
-		if (PD(bop)->remove_original && ! same_uri (fd->path, PD(bop)->new_path)) {
-			file_unlink (fd->path);
-			PD(bop)->deleted_list = g_list_prepend (PD(bop)->deleted_list, g_strdup (fd->path));
+		if (! same_uri (fd->path, PD(bop)->new_path)) {
+			comment_copy (fd->path, PD(bop)->new_path);
+			if (PD(bop)->remove_original) {
+				file_unlink (fd->path);
+				PD(bop)->deleted_list = g_list_prepend (PD(bop)->deleted_list, g_strdup (fd->path));
+			}
 		}
-	} else 
+
+	} else
 		_gtk_error_dialog_from_gerror_run (PD(bop)->parent, &error);
 
 	load_next_image (bop);
@@ -477,7 +484,7 @@ save_image_and_remove_original (GthBatchOp *bop)
 	else
 		dest = src;
 	gth_pixbuf_op_set_pixbufs (PD(bop)->pixbuf_op, src, dest);
-	gth_pixbuf_op_start (PD(bop)->pixbuf_op); 
+	gth_pixbuf_op_start (PD(bop)->pixbuf_op);
 }
 
 
@@ -508,9 +515,9 @@ rename_response_cb (GtkWidget  *dialog,
 	g_free (folder);
 	g_free (new_name);
 
-	if (path_is_file (PD(bop)->new_path)) 
+	if (path_is_file (PD(bop)->new_path))
 		show_rename_dialog (bop);
-	else 
+	else
 		save_image_and_remove_original (bop);
 }
 
@@ -522,7 +529,7 @@ overwrite_response_cb (GtkWidget  *dialog,
 {
 	gtk_widget_destroy (dialog);
 
-	if (response_id == GTK_RESPONSE_YES) 
+	if (response_id == GTK_RESPONSE_YES)
 		save_image_and_remove_original (bop);
 	else
 		load_next_image (bop);
@@ -559,7 +566,7 @@ loader_done (ImageLoader *il,
 			break;
 
 		case GTH_OVERWRITE_ASK:
-			utf8_name = g_filename_display_basename (PD(bop)->new_path);
+			utf8_name = basename_for_display (PD(bop)->new_path);
 			message = g_strdup_printf (_("An image named \"%s\" is already present. " "Do you want to overwrite it?"), utf8_name);
 
 			d = _gtk_yesno_dialog_new (GTK_WINDOW (PD(bop)->progress_dlg),
@@ -577,17 +584,17 @@ loader_done (ImageLoader *il,
 
 			gtk_widget_show (d);
 			return;
-			
+
 		case GTH_OVERWRITE_RENAME:
 			show_rename_dialog (bop);
 			return;
 		}
-	} 
+	}
 
 	if (save_image)
 		save_image_and_remove_original (bop);
 	else
-		load_next_image (bop);	
+		load_next_image (bop);
 }
 
 
@@ -600,8 +607,8 @@ loader_error (ImageLoader *il,
 
 
 static gboolean
-progress_dlg_delete_event_cb (GtkWidget  *caller, 
-			      GdkEvent   *event, 
+progress_dlg_delete_event_cb (GtkWidget  *caller,
+			      GdkEvent   *event,
 			      GthBatchOp *bop)
 {
 	notify_termination (bop);
@@ -610,8 +617,8 @@ progress_dlg_delete_event_cb (GtkWidget  *caller,
 
 
 static gboolean
-rename_dlg_delete_event_cb (GtkWidget  *caller, 
-			    GdkEvent   *event, 
+rename_dlg_delete_event_cb (GtkWidget  *caller,
+			    GdkEvent   *event,
 			    GthBatchOp *bop)
 {
 	return TRUE;
@@ -643,7 +650,7 @@ gth_batch_op_start (GthBatchOp       *bop,
 			  "image_error",
 			  G_CALLBACK (loader_error),
 			  bop);
-	
+
 	/**/
 
 	PD(bop)->gui = glade_xml_new (GTHUMB_GLADEDIR "/" CONVERT_GLADE_FILE, NULL, NULL);
@@ -669,16 +676,16 @@ gth_batch_op_start (GthBatchOp       *bop,
 
 	progress_cancel = glade_xml_get_widget (PD(bop)->progress_gui, "p_cancelbutton");
 
-	gtk_window_set_transient_for (GTK_WINDOW (PD(bop)->rename_dialog), 
+	gtk_window_set_transient_for (GTK_WINDOW (PD(bop)->rename_dialog),
 				      GTK_WINDOW (PD(bop)->progress_dlg));
 
 	/* Set the signals handlers. */
-	
-	g_signal_connect (G_OBJECT (progress_cancel), 
+
+	g_signal_connect (G_OBJECT (progress_cancel),
 			  "clicked",
 			  G_CALLBACK (stop_operation_cb),
 			  bop);
-	g_signal_connect (G_OBJECT (PD(bop)->rename_dialog), 
+	g_signal_connect (G_OBJECT (PD(bop)->rename_dialog),
 			  "response",
 			  G_CALLBACK (rename_response_cb),
 			  bop);
@@ -703,17 +710,17 @@ gth_batch_op_start (GthBatchOp       *bop,
 	PD(bop)->images = g_list_length (PD(bop)->file_list);
 	PD(bop)->image = 1;
 
-	if (!dlg_save_options (GTK_WINDOW (PD(bop)->parent), 
+	if (!dlg_save_options (GTK_WINDOW (PD(bop)->parent),
 			       PD(bop)->image_type,
-			       &(PD(bop)->keys), 
+			       &(PD(bop)->keys),
 			       &(PD(bop)->values))) {
 		free_progress_data (bop);
 		return;
 	}
 
-	gtk_window_set_transient_for (GTK_WINDOW (PD(bop)->progress_dlg), 
+	gtk_window_set_transient_for (GTK_WINDOW (PD(bop)->progress_dlg),
 				      GTK_WINDOW (PD(bop)->parent));
-	gtk_window_set_modal (GTK_WINDOW (PD(bop)->progress_dlg), TRUE); 
+	gtk_window_set_modal (GTK_WINDOW (PD(bop)->progress_dlg), TRUE);
 	gtk_widget_show (PD(bop)->progress_dlg);
 
 	if (PD(bop)->pixbuf_op->single_step)
